/* graph.h - Morse Code Dojo graph widget
 * 
 *  Morse Code Dojo
 * 
 *  Copyright (c) 2001 Kevin Worcester
 * 
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, 
 *  USA.
 */
#ifndef GRAPH_H
#define GRAPH_H

#include <qwidget.h>
#include <vector.h>

#define MAX_GRID_SIZE 43

class Graph  : public QWidget
{
   Q_OBJECT
public:
   typedef vector<int> GraphElementList;

   Graph(QWidget *parent = NULL, const char *name = NULL );
   virtual ~Graph();
   void addPoint(int x, int y);
   void setScale(int min, int max);
   void setScale(int max);
   bool isClickable(void);
public slots:
   void reset(void);
   void showGraph(void);
   void setClickable(bool o);
signals:
   void clickedAt(int x, int y);
protected slots:
   void paintEvent( QPaintEvent *e );
   void mousePressEvent( QMouseEvent *e );
   void mouseMoveEvent( QMouseEvent *e );
protected:
   QColor m_graphColor;
   int m_list[MAX_GRID_SIZE];
private:
   int  m_min;
   int  m_max;
   bool m_clickable;
};

#endif /* GRAPH_H */
