/* lesson.h - Morse Code Dojo lesson widget
 * 
 *  Morse Code Dojo
 * 
 *  Copyright (c) 2001 Kevin Worcester
 * 
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, 
 *  USA.
 */
#ifndef LESSON_H
#define LESSON_H

#include <qvariant.h>
#include <qdialog.h>
#include <sys/types.h>
#include <time.h>
#include <stdio.h>

enum LessonState {
   LS_None,
   LS_Pause,
   LS_Teach,
   LS_Random,
   LS_Word,
   LS_End
};

class QVBoxLayout; 
class QHBoxLayout; 
class QGridLayout; 
class QFrame;
class QLabel;
class QPushButton;
class QTimer;
class Morse;
class MRand;

class Lesson : public QDialog
{ 
   Q_OBJECT

public:
   Lesson( QWidget* parent = 0, const char* name = 0, bool modal = FALSE, WFlags fl = 0 );
   ~Lesson();
   void show();

protected slots:
   void startPressed();
   void stopPressed();
   void resendTimeout();
   void elapsedTimeout();
   void closeEvent(QCloseEvent *e);
   void setLesson(int l);

signals:
   void closed();

protected:
   void keyPressEvent(QKeyEvent *e);
   void doLine(void);

   int m_current;
   int m_max;
   int m_lastWrong;
   int m_lesson;
   int m_timeRemaining;

   LessonState m_state;

   FILE *m_fp;

   QString m_sent;
   Morse *m_morse;
   MRand *m_mRand;
   time_t m_startTime;
   time_t m_maxTime;

   QFrame* Frame5;
   QFrame* Frame4;
   QLabel* sent_v;
   QLabel* message_v;

   QTimer      * resend_t;
   QTimer      * elapsed_t;
   QPushButton* start_b;
   QPushButton* stop_b;
   QPushButton* close_b;
   QVBoxLayout* LessonLayout;
   QVBoxLayout* Frame5Layout;
   QHBoxLayout* Layout5;
   QHBoxLayout* Layout2;
   QHBoxLayout* Layout3;
   QHBoxLayout* Layout4;
   QHBoxLayout* Layout1;
   QVBoxLayout* Frame4Layout;
   QHBoxLayout* Layout8;
   bool event( QEvent* );
};

#endif // LESSON_H

