/* morse.h - Morse Code Dojo morse code object
 * 
 *  Morse Code Dojo
 * 
 *  Copyright (c) 2001 Kevin Worcester
 * 
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, 
 *  USA.
 */
#ifndef MORSE
#define MORSE

#include <stdio.h>
#include <stdlib.h>
#include <qwidget.h>
#include <qtimer.h>

#define SEND_QUEUE_SIZE  4096
#define MORSE_8KHZ  8192
#define MORSE_16KHZ 16384
#define MORSE_RATE  MORSE_8KHZ

const char morse_char[96][15] = {
   "", "", "", "", "", "", "", "",
   "", "", "", "", "", "", "", "",
   "", "", "", "", "", "", "", "",
   "", "", "", "", "", "", "", "",
   " ",      //    (space)
   "",        //  !
   "",        //  "
   "",        //  #
   "",        //  $
   "",        //  %
   "",        //  &
   "",        //  '
   "",        //  (
   "",        //  )
   "...-.-",  //  * (SK)
   ".-.-.",   //  + (AR)
   "--..--",  //  ,
   "",        //  -
   ".-.-.-",  //  .
   "-..-.",   //  /
   "-----",   //  0
   ".----",   //  1
   "..---",   //  2
   "...--",   //  3
   "....-",   //  4
   ".....",   //  5
   "-....",   //  6
   "--...",   //  7
   "---..",   //  8
   "----.",   //  9
   "",        //  :
   "",        //  ;
   "",        //  <
   "-...-",   //  =
   "",        //  >
   "..--..",  //  ?
   "",        //  @
   ".-",      //  A
   "-...",    //  B
   "-.-.",    //  C
   "-..",     //  D
   ".",       //  E
   "..-.",    //  F
   "--.",     //  G
   "....",    //  H
   "..",      //  I
   ".---",    //  J
   "-.-",     //  K
   ".-..",    //  L
   "--",      //  M
   "-.",      //  N
   "---",     //  O
   ".--.",    //  P
   "--.-",    //  Q
   ".-.",     //  R
   "...",     //  S
   "-",       //  T
   "..-",     //  U
   "...-",    //  V
   ".--",     //  W
   "-..-",    //  X
   "-.--",    //  Y
   "--..",    //  Z
};

class Morse : public QWidget {
   Q_OBJECT
 public:
   ~Morse();

   static Morse * getInstance();

   void openDevice();
   void flushDevice();

   void correct();
   void incorrect();

   bool setSpeed(unsigned charspeed, unsigned wordspeed);
   bool setCharacterSpeed(unsigned);
   bool setWordSpeed(unsigned);

   bool setCharacterTone(unsigned);
   unsigned characterSpeed();
   unsigned wordSpeed();
   unsigned characterTone();

   long length( char ch );

   void sendInternal(char);
   void send(char);
   void send(const char *);
   void sendSpace(const char *);
   bool canSend();
   bool sending();

   void abort();

 protected slots:
   void sendTimeout(void);

 protected:
   Morse(unsigned char_speed = 17, unsigned word_speed = 8, unsigned tone = 720);
   bool queue(char ch);
   bool queue(const char *s);

   FILE *m_fp;  /* Sound file handles */
   int   m_fd;

   int    m_charWpm;  /* Character WPM  (defautls to 17)  */
   int    m_wordWpm;  /* Character WPM  (defautls to 17)  */
   double m_charTick; /* Character tick, time of one dit (in fractions of a second) */
   double m_wordTick; /* Word tick, time of one dit (in fractions of a second) */
   int    m_charDur;  /* Bytes of sound buffer = 1 tick for characters */
   int    m_wordDur;  /* Bytes of sound buffer = 1 tick for words */
   int    m_tone;     /* Tone frequency (defaults to 720 (Hz) */

   int    m_maxBytes; /* Maximum bytes writable without blocking */

   unsigned char m_buf[MORSE_RATE];      /* Tone buffer */
   unsigned char m_silence[MORSE_RATE];  /* Silence buffer */
   unsigned char m_buzz[MORSE_RATE];     /* Incorrect buzzer */
   unsigned char m_bell[MORSE_RATE];  /* Correct bell */

   char m_sendQueue[SEND_QUEUE_SIZE];

   int m_queueLocation;

   static Morse * s_instance;

   QTimer *send_t;
};

#endif /* MORSE */
