/* mrand.cpp - Morse Code Dojo random code generator
 * 
 *  Morse Code Dojo
 * 
 *  Copyright (c) 2001 Kevin Worcester
 * 
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, 
 *  USA.
 */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <ctype.h>
#include "mrand.h"

MRand * MRand::s_instance = NULL;

Weight WHistory;
Weight WText;
Weight WStudent;


struct MorseCharacter {
   char m_char;
   unsigned int  m_weight;
};

struct MorseCharacter char_pool[43] = {
   {'A', 1},
   {'B', 1},
   {'C', 1},
   {'D', 1},
   {'E', 1},
   {'F', 1},
   {'G', 1},
   {'H', 1},
   {'I', 1},
   {'J', 1},
   {'K', 1},
   {'L', 1},
   {'M', 1},
   {'N', 1},
   {'O', 1},
   {'P', 1},
   {'Q', 1},
   {'R', 1},
   {'S', 1},
   {'T', 1},
   {'U', 1},
   {'V', 1},
   {'W', 1},
   {'X', 1},
   {'Y', 1},
   {'Z', 1},
   {'0', 1},
   {'1', 1},
   {'2', 1},
   {'3', 1},
   {'4', 1},
   {'5', 1},
   {'6', 1},
   {'7', 1},
   {'8', 1},
   {'9', 1},
   {'+', 1},
   {'*', 1},
   {'/', 1},
   {'.', 1},
   {',', 1},
   {'?', 1},
   {'=', 1}
};


MRand::MRand()
{
   srand(time(NULL));

   m_maxCharacter = 0;
   for ( int t=0; t < 43; t++ )
   {
      m_maxCharacter += char_pool[t].m_weight;
   }

   m_minStrLen = 3;
   m_maxStrLen = 10;

   WStudent.setAll(1);
   WHistory.setAll(1);
   WText.setAll(1);

   setWeight( MRRandom );
}

MRand::~MRand()
{
}

MRand * MRand::getInstance(void)
{
   if ( s_instance == NULL )
   {
      s_instance = new MRand();
   }

   return s_instance;
}

char MRand::weightToChar(int w)
{
   for ( int t=0; t < 43; t++)
   {
      if ( char_pool[t].m_weight )
      {
         w -= char_pool[t].m_weight;
         if ( w <= 0 )
         {
            return char_pool[t].m_char;
         }
      }
   }
   return ' ';
}

char MRand::intToChar(int n)
{
   return char_pool[n].m_char;
}

int MRand::charToInt(char ch)
{
   for ( int t=0; t < 43; t++)
   {
      if ( char_pool[t].m_char == ch )
      {
         return t;
      }
   }
   return -1;
}


char MRand::getCharacter()
{
   if( m_maxCharacter )
      return( weightToChar(random() % (m_maxCharacter + 1)));
   else
      return( ' ' );
}

char * MRand::getString()
{
   int chars;
   char *str = strdup("          ");

   if( m_maxStrLen != m_minStrLen )
      chars = m_minStrLen + ( random() % (m_maxStrLen - m_minStrLen +1) );
   else
      chars = m_maxStrLen;

   int t;
   for ( t=0; t<chars; t++ )
   {
      str[t] = getCharacter();
   }
   str[t] = '\0';

   return(str);
}

void MRand::setWordRange(int min, int max)
{
   if ( min < 1 )
      min = 1;
   if ( max > 10 )
      max = 10;
   if ( max < min )
      max = min;

   m_maxStrLen = max;
   m_minStrLen = min;
}

void MRand::setMaxLength(int max)
{
   if ( max > 10 )
      max = 10;
   if ( max < m_minStrLen )
      m_minStrLen = max;

   m_maxStrLen = max;
}

void MRand::setMinLength(int min)
{
   if ( min < 1 )
      min = 1;
   if ( m_maxStrLen < min )
      m_maxStrLen = min;

   m_minStrLen = min;
}

int MRand::maxLength()
{
   return m_maxStrLen;
}

int MRand::minLength()
{
   return m_minStrLen;
}

void MRand::setCharWeight(char ch, int w)
{
   if ( w < 0)
      return;

   for ( int t=0; t<43; t++ )
   {
      if ( ch == char_pool[t].m_char )
      {
         m_maxCharacter -= char_pool[t].m_weight;
         m_maxCharacter += w;
         char_pool[t].m_weight = w;
         break;
      }
   }
}

void MRand::setWeight( Weight & w )
{
   m_weight = MRStudent;
   m_maxCharacter = 0;
   for ( int t = 0; t < 43; t++)
   {
      m_maxCharacter         += w.weight(t);
      char_pool[t].m_weight   = w.weight(t);
   }
}

MRWeight MRand::currentWeight( void )
{
   return m_weight;
}

void MRand::getWeight( Weight & w )
{
   for ( int t = 0; t < 43; t++)
   {
      w.setWeight(t, char_pool[t].m_weight );
   }
}

void MRand::setWeight( MRWeight w )
{
   m_weight = w;
   switch ( w )
   {
      case MRRandom:

         m_maxCharacter = 43;
         for ( int t = 0; t < 43; t++)
         {
            char_pool[t].m_weight = 1;
         }

         break;
      case MRHistory:
         m_maxCharacter = 0;
         for ( int t = 0; t < 43; t++)
         {
            m_maxCharacter         += WHistory.weight(t);
            char_pool[t].m_weight   = WHistory.weight(t);
         }
         break;

      case MRStudent:
         m_maxCharacter = 0;
         for ( int t = 0; t < 43; t++)
         {
            m_maxCharacter         += WStudent.weight(t);
            char_pool[t].m_weight   = WStudent.weight(t);
         }
         break;

      case MRLetters:

         m_maxCharacter = 0;
         for ( int t = 0; t < 43; t++)
         {
            if( isalpha(char_pool[t].m_char) )
            {
               char_pool[t].m_weight = 1;
               m_maxCharacter++;
            }
            else
            {
               char_pool[t].m_weight = 0;
            }
         }


         break;
      case MRNumbers:

         m_maxCharacter = 0;
         for ( int t = 0; t < 43; t++)
         {
            if( isdigit(char_pool[t].m_char) )
            {
               char_pool[t].m_weight = 1;
               m_maxCharacter++;
            }
            else
            {
               char_pool[t].m_weight = 0;
            }
         }


         break;
      case MRText:
         m_maxCharacter = 0;
         for ( int t = 0; t < 43; t++)
         {
            m_maxCharacter         += WText.weight(t);
            char_pool[t].m_weight   = WText.weight(t);
         }
         break;

      default:
         fprintf(stderr, "Unknown weight value\n");
         break;
   }
}

int MRand::getCharWeight(char ch)
{
   for ( int t=0; t<43; t++ )
   {
      if ( ch == char_pool[t].m_char )
      {
         return(char_pool[t].m_weight);
      }
   }

   return(0);
}

void MRand::clearWeight(void)
{
   m_maxCharacter = 0;
   for ( int t=0; t<43; t++ )
   {
      char_pool[t].m_weight = 0;
   }
}

void MRand::setAll(void)
{
   m_maxCharacter = 43;
   for ( int t=0; t<43; t++ )
   {
      char_pool[t].m_weight = 1;
   }
}

int MRand::getAbsWeight( int t )
{
   return ( char_pool[t].m_weight );
}

void MRand::setAbsWeight( int t, int w )
{
   char_pool[t].m_weight = w;
}

