/* practice.cpp - Morse Code Dojo practice window
 * 
 *  Morse Code Dojo
 * 
 *  Copyright (c) 2001 Kevin Worcester
 * 
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, 
 *  USA.
 */
#include <qframe.h>
#include <qlabel.h>
#include <qpushbutton.h>
#include <qlayout.h>
#include <qvariant.h>
#include <qtooltip.h>
#include <qwhatsthis.h>
#include <qtimer.h>
#include <ctype.h>
#include <unistd.h>

#include "practice.h"
#include "morse.h"
#include "mrand.h"

FTime::FTime(time_t sec, int ms)
{
   m_sec = sec;
   m_ms  = ms;
}

FTime::~FTime()
{
}

void FTime::setTime(time_t sec, int ms)
{
   m_sec = sec;
   m_ms  = ms;
}

void FTime::getCurrentTime(void)
{
   struct timeb t;
   ftime( &t );
   m_sec = t.time;
   m_ms  = t.millitm;
}

long operator-(FTime & f1, FTime & f2)
{
   int    ms;
   time_t sec;

   ms  = f1.ms()      - f2.ms();
   sec = f1.seconds() - f2.seconds();

   while ( ms < 0 )
   {
      ms += 1000;
      sec--;
   }

   if ( sec < 0 )
   {
      sec = 0;
      ms  = 0;
   }

   return ( (long) sec * 1000 + ms );
}

long operator+(FTime &f1, FTime &f2)
{
   int    ms;
   time_t sec;

   ms  = f1.ms()      + f2.ms();
   sec = f1.seconds() + f2.seconds();

   while ( ms >= 1000 )
   {
      ms -= 1000;
      sec++;
   }

   return ( (long) sec * 1000 + ms );
}

/* 
 *  Constructs a Practice which is a child of 'parent', with the 
 *  name 'name' and widget flags set to 'f' 
 *
 *  The dialog will by default be modeless, unless you set 'modal' to
 *  TRUE to construct a modal dialog.
 */
Practice::Practice( QWidget* parent,  const char* name, bool modal, WFlags fl )
   : QDialog( parent, name, modal, fl )
{
   if ( !name )
   setName( "Practice" );
   resize( 575, 450 ); 
   setCaption( tr( "Practice" ) );
   PracticeLayout = new QVBoxLayout( this ); 
   PracticeLayout->setSpacing( 6 );
   PracticeLayout->setMargin( 11 );

   Frame5 = new QFrame( this, "Frame5" );
   Frame5->setSizePolicy( QSizePolicy( (QSizePolicy::SizeType)5, (QSizePolicy::SizeType)0, Frame5->sizePolicy().hasHeightForWidth() ) );
   Frame5->setFrameShape( QFrame::Box );
   Frame5->setFrameShadow( QFrame::Sunken );
   Frame5Layout = new QVBoxLayout( Frame5 ); 
   Frame5Layout->setSpacing( 6 );
   Frame5Layout->setMargin( 11 );

   Layout5 = new QHBoxLayout; 
   Layout5->setSpacing( 6 );
   Layout5->setMargin( 0 );

   Layout2 = new QHBoxLayout; 
   Layout2->setSpacing( 6 );
   Layout2->setMargin( 0 );

   correct_l = new QLabel( Frame5, "correct_l" );
   correct_l->setText( tr( "Correct:" ) );
   Layout2->addWidget( correct_l );

   correct_v = new QLabel( Frame5, "correct_v" );
   correct_v->setText( tr( "0" ) );
   Layout2->addWidget( correct_v );
   Layout5->addLayout( Layout2 );

   Layout3 = new QHBoxLayout; 
   Layout3->setSpacing( 6 );
   Layout3->setMargin( 0 );

   incorrect_l = new QLabel( Frame5, "incorrect_l" );
   incorrect_l->setText( tr( "Incorrect:" ) );
   Layout3->addWidget( incorrect_l );

   incorrect_v = new QLabel( Frame5, "incorrect_v" );
   incorrect_v->setText( tr( "0" ) );
   Layout3->addWidget( incorrect_v );
   Layout5->addLayout( Layout3 );

   Layout4 = new QHBoxLayout; 
   Layout4->setSpacing( 6 );
   Layout4->setMargin( 0 );

   percent_l = new QLabel( Frame5, "percent_l" );
   percent_l->setText( tr( "Percent:" ) );
   Layout4->addWidget( percent_l );

   percent_v = new QLabel( Frame5, "percent_v" );
   percent_v->setText( tr( "0.0%" ) );
   Layout4->addWidget( percent_v );
   Layout5->addLayout( Layout4 );
   Frame5Layout->addLayout( Layout5 );

   Layout1 = new QHBoxLayout; 
   Layout1->setSpacing( 6 );
   Layout1->setMargin( 0 );

   time_l = new QLabel( Frame5, "time_l" );
   time_l->setText( tr( "Time elapsed:" ) );
   time_l->setAlignment( int( QLabel::AlignVCenter | QLabel::AlignRight ) );
   Layout1->addWidget( time_l );

   time_v = new QLabel( Frame5, "time_v" );
   time_v->setText( tr( "0:00" ) );
   Layout1->addWidget( time_v );
   Frame5Layout->addLayout( Layout1 );
   PracticeLayout->addWidget( Frame5 );

   Frame4 = new QFrame( this, "Frame4" );
   Frame4->setFrameShape( QFrame::Box );
   Frame4->setFrameShadow( QFrame::Sunken );
   Frame4Layout = new QVBoxLayout( Frame4 ); 
   Frame4Layout->setSpacing( 6 );
   Frame4Layout->setMargin( 11 );
   QSpacerItem* spacer = new QSpacerItem( 20, 20, QSizePolicy::Minimum, QSizePolicy::Expanding );
   Frame4Layout->addItem( spacer );

   sent_v = new QLabel( Frame4, "sent_v" );
   sent_v->setSizePolicy( QSizePolicy( (QSizePolicy::SizeType)5, (QSizePolicy::SizeType)5, sent_v->sizePolicy().hasHeightForWidth() ) );
   QFont sent_v_font(  sent_v->font() );
   sent_v_font.setPointSize( 20 );
   sent_v->setFont( sent_v_font ); 
   sent_v->setText( "" );
   sent_v->setAlignment( int( QLabel::AlignCenter ) );
   Frame4Layout->addWidget( sent_v );

   message_v = new QLabel( Frame4, "message_v" );
   QFont message_v_font(  message_v->font() );
   message_v_font.setPointSize( 20 );
   message_v->setFont( message_v_font ); 
   message_v->setText( "" );
   message_v->setAlignment( int( QLabel::AlignCenter ) );
   Frame4Layout->addWidget( message_v );

   QSpacerItem* spacer_2 = new QSpacerItem( 20, 20, QSizePolicy::Minimum, QSizePolicy::Expanding );
   Frame4Layout->addItem( spacer_2 );
   PracticeLayout->addWidget( Frame4 );

   Layout8 = new QHBoxLayout; 
   Layout8->setSpacing( 6 );
   Layout8->setMargin( 0 );

   start_b = new QPushButton( this, "start_b" );
   start_b->setText( tr( "Start" ) );
   Layout8->addWidget( start_b );

   stop_b = new QPushButton( this, "stop_b" );
   stop_b->setText( tr( "Stop" ) );
   Layout8->addWidget( stop_b );

   close_b = new QPushButton( this, "close_b" );
   close_b->setText( tr( "Close" ) );
   Layout8->addWidget( close_b );
   PracticeLayout->addLayout( Layout8 );

   resend_t  = new QTimer(this, "");
   elapsed_t = new QTimer(this, "");

   // signals and slots connections
   connect( close_b,   SIGNAL( clicked() ), this, SLOT( close() ) );
   connect( start_b,   SIGNAL( clicked() ), this, SLOT( startPressed() ) );
   connect( stop_b,    SIGNAL( clicked() ), this, SLOT( stopPressed() ) );
   connect( resend_t,  SIGNAL( timeout() ), this, SLOT( resendTimeout() ) );
   connect( elapsed_t, SIGNAL( timeout() ), this, SLOT( elapsedTimeout() ) );

   m_morse = Morse::getInstance();
   m_mRand = MRand::getInstance();
   m_accepting = false;
   m_maxTime   = 60;
}

Practice::~Practice()
{
   delete m_morse;
}

void Practice::closeEvent(QCloseEvent *e)
{
   m_morse->abort();
   resend_t->stop();
   elapsed_t->stop();

   emit closed();
   QDialog::closeEvent(e);
}

void Practice::show()
{
   m_morse->abort();
   elapsed_t->stop();
   resend_t->stop();

   m_correct   = 0;
   m_incorrect = 0;
   m_current   = 0;
   m_lastWrong = 0;
   m_sent      = "";
   m_max       = -1;
   m_accepting = false;
   
   correct_v->setText("0");
   incorrect_v->setText("0");
   percent_v->setText("0.0%");
   time_v->setText("0:00");
   sent_v->setText("<html>");
   message_v->setText("<html>");

   QDialog::show();
}

/*  
 *  Main event handler. Reimplemented to handle application
 *  font changes
 */
bool Practice::event( QEvent* ev )
{
   bool ret = QDialog::event( ev ); 
   if ( ev->type() == QEvent::ApplicationFontChange ) {
      QFont sent_v_font(  sent_v->font() );
      sent_v_font.setPointSize( 20 );
      sent_v->setFont( sent_v_font ); 
      QFont message_v_font(  message_v->font() );
      message_v_font.setPointSize( 20 );
      message_v->setFont( message_v_font ); 
   }
   return ret;
}

void Practice::keyPressEvent(QKeyEvent *e)
{
   int diff;

   char ch = e->ascii();
   ch = toupper(ch);
   if ( m_accepting && (ch >= ' ' && ch <='~') )
   {
      QString tempNum;
      if( (char) (QChar) m_sent[m_current] == ch)
      {
         FTime currentTime;
         currentTime.getCurrentTime();

         m_charCount[m_mRand->charToInt((char) (QChar) m_sent[m_current])]++;

         diff = (int) (currentTime - m_sendTime);
         
         if ( diff < 0 )
            diff = 0;

         m_charTime[m_mRand->charToInt(ch)] += diff;

         m_correct++;
         tempNum.setNum(m_correct);
         correct_v->setText(tempNum);

         tempNum.sprintf("%.1f%%", m_correct * 100.0 / (m_incorrect + m_correct) );
         percent_v->setText(tempNum);

         if ( ! m_lastWrong )
         {
            sent_v->setText( tr( sent_v->text().append(QString( tr(" ") + QChar(ch))) ) );
         }
         else
         {
            message_v->setText("");
         }
         m_lastWrong = 0;

         m_sent[++m_current]   = m_mRand->getCharacter();
         m_morse->send( (char) (QChar) m_sent[m_current] );
         m_sendTime.getCurrentTime();
      }
      else
      {
         m_charWrong[m_mRand->charToInt((char) (QChar) m_sent[m_current])]++;

         m_accepting = false;
         m_incorrect++;
         tempNum.setNum(m_incorrect);
         incorrect_v->setText(tempNum);

         tempNum.sprintf("%.1f%%", m_correct * 100.0 / (m_incorrect + m_correct) );
         percent_v->setText(tempNum);

         if ( ! m_lastWrong) 
         {
            sent_v->setText( tr( sent_v->text().append(QString( tr(" <font color=\"red\">") + m_sent[m_current])) + tr("</font>") ) );
         }
         message_v->setText( tr("<html><font color=\"red\">") + QChar(ch) + tr("</font>") );
         m_lastWrong = 1;
         m_morse->incorrect();

         resend_t->start(2000, TRUE);
      }
   }
   else
   {
      QDialog::keyPressEvent(e);
   }
}

void Practice::stopPressed(void)
{
   m_morse->abort();
   elapsed_t->stop();
   resend_t->stop();
   m_accepting = false;
}

void Practice::startPressed(void)
{
   m_morse->abort();
   elapsed_t->stop();
   resend_t->stop();

   for ( int t = 0; t<43; t++ )
   {
      m_charCount[t] = 0;
      m_charWrong[t] = 0;
      m_charTime[t]  = 0;
   }

   m_correct   = 0;
   m_incorrect = 0;
   m_current   = 0;
   m_lastWrong = 0;
   m_sent      = "";
   m_max       = -1;
   m_accepting = true;
   
   correct_v->setText("0");
   incorrect_v->setText("0");
   sent_v->setText("<html>");
   message_v->setText("<html>");

   m_sent[0]   = m_mRand->getCharacter();

   m_startTime = time(NULL);
   m_morse->send( (char) (QChar) m_sent[m_current] );
   m_sendTime.getCurrentTime();
   elapsed_t->start(1000, FALSE);
}

void Practice::resendTimeout(void)
{
   m_max       = -1;
   m_accepting = true;
   
   message_v->setText("<html>");

   m_morse->send( (char) (QChar) m_sent[m_current] );
//   m_sendTime.getCurrentTime();
}

void Practice::elapsedTimeout(void)
{
   int min, sec, t;
   t = time(NULL) - m_startTime;

   if ( t >= m_maxTime )
   {
      elapsed_t->stop();
      resend_t->stop();

      m_morse->abort();
      m_accepting = false;

      message_v->setText("Time up");
   }
   min = t / 60;
   sec = t % 60;

   QString timestr;
   timestr.sprintf("%d:%02d", min, sec);
   time_v->setText(timestr);
}

void Practice::setTime(time_t t)
{
   m_maxTime = t;
}

time_t Practice::getTime(void)
{
   return m_maxTime;
}

void Practice::reject()
{
   emit closed();
   QDialog::reject();
}

