/* practice.h - Morse Code Dojo practice window
 * 
 *  Morse Code Dojo
 * 
 *  Copyright (c) 2001 Kevin Worcester
 * 
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, 
 *  USA.
 */
#ifndef PRACTICE_H
#define PRACTICE_H

#include <qvariant.h>
#include <qdialog.h>
#include <time.h>
#include <sys/types.h>
#include <sys/timeb.h>

class QVBoxLayout; 
class QHBoxLayout; 
class QGridLayout; 
class QFrame;
class QLabel;
class QPushButton;
class QTimer;
class Morse;
class MRand;

class FTime {
public:
   FTime(time_t sec = 0, int ms = 0);
   virtual ~FTime();
   void getCurrentTime(void);
   void setTime(time_t sec, int ms);

   time_t seconds()
   {
      return m_sec;
   }
   int ms()
   {
      return m_ms;
   }
   long total()
   {
      return ((long) m_sec * 1000 + m_ms);
   }

protected:
   int    m_ms;
   time_t m_sec;
};

FTime operator-(const FTime &, const FTime &);
FTime operator+(const FTime &, const FTime &);

class Practice : public QDialog
{ 
   Q_OBJECT

public:
   Practice( QWidget* parent = 0, const char* name = 0, bool modal = FALSE, WFlags fl = 0 );
   virtual ~Practice();
   void show();

   void   setTime(time_t t);
   time_t getTime();

protected slots:
   void reject();
   void startPressed();
   void stopPressed();
   void resendTimeout();
   void elapsedTimeout();
   void closeEvent(QCloseEvent *e);

signals:
   void closed();

protected:
   void keyPressEvent(QKeyEvent *e);

   int m_correct;
   int m_incorrect;
   int m_current;
   int m_max;
   int m_lastWrong;
   QString m_sent;
   Morse *m_morse;
   MRand *m_mRand;
   bool m_accepting;
   time_t m_startTime;
   time_t m_maxTime;
   int m_charWrong[43];
   int m_charTime[43];
   int m_charCount[43];
   FTime m_sendTime;

   QFrame* Frame5;
   QLabel* correct_l;
   QLabel* correct_v;
   QLabel* incorrect_l;
   QLabel* incorrect_v;
   QLabel* percent_l;
   QLabel* percent_v;
   QLabel* time_l;
   QLabel* time_v;
   QFrame* Frame4;
   QLabel* sent_v;
   QLabel* message_v;

   QTimer      * resend_t;
   QTimer      * elapsed_t;
   QPushButton* start_b;
   QPushButton* stop_b;
   QPushButton* close_b;
   QVBoxLayout* PracticeLayout;
   QVBoxLayout* Frame5Layout;
   QHBoxLayout* Layout5;
   QHBoxLayout* Layout2;
   QHBoxLayout* Layout3;
   QHBoxLayout* Layout4;
   QHBoxLayout* Layout1;
   QVBoxLayout* Frame4Layout;
   QHBoxLayout* Layout8;
   bool event( QEvent* );
};

#endif // PRACTICE_H
