/* prof.cpp - Morse Code Dojo proficiency window
 * 
 *  Morse Code Dojo
 * 
 *  Copyright (c) 2001 Kevin Worcester
 * 
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, 
 *  USA.
 */
#include "prof.h"

#include <qframe.h>
#include <qlabel.h>
#include <qmultilineedit.h>
#include <qtextview.h>
#include <qpushbutton.h>
#include <qlayout.h>
#include <qvariant.h>
#include <qtooltip.h>
#include <qwhatsthis.h>
#include <qtimer.h>
#include <ctype.h>
#include <qtextbrowser.h>

#include "morse.h"
#include "mrand.h"

/* 
 *  Constructs a Prof which is a child of 'parent', with the 
 *  name 'name' and widget flags set to 'f' 
 *
 *  The dialog will by default be modeless, unless you set 'modal' to
 *  TRUE to construct a modal dialog.
 */
Prof::Prof( QWidget* parent,  const char* name, bool modal, WFlags fl )
   : QDialog( parent, name, modal, fl )
{
   if ( !name )
      setName( "Prof" );
   resize( 596, 480 ); 
   setCaption( tr( "Sample QSO" ) );
   ProfLayout = new QVBoxLayout( this ); 
   ProfLayout->setSpacing( 6 );
   ProfLayout->setMargin( 11 );

   Frame5 = new QFrame( this, "Frame5" );
   Frame5->setSizePolicy( QSizePolicy( (QSizePolicy::SizeType)5, (QSizePolicy::SizeType)0, Frame5->sizePolicy().hasHeightForWidth() ) );
   Frame5->setFrameShape( QFrame::Box );
   Frame5->setFrameShadow( QFrame::Sunken );
   Frame5Layout = new QVBoxLayout( Frame5 ); 
   Frame5Layout->setSpacing( 6 );
   Frame5Layout->setMargin( 11 );

   Layout1 = new QHBoxLayout; 
   Layout1->setSpacing( 6 );
   Layout1->setMargin( 0 );

   time_l = new QLabel( Frame5, "time_l" );
   time_l->setText( tr( "Time elapsed:" ) );
   time_l->setAlignment( int( QLabel::AlignVCenter | QLabel::AlignRight ) );
   Layout1->addWidget( time_l );

   time_v = new QLabel( Frame5, "time_v" );
   time_v->setText( tr( "0:00" ) );
   Layout1->addWidget( time_v );
   Frame5Layout->addLayout( Layout1 );
   ProfLayout->addWidget( Frame5 );

   Frame4 = new QFrame( this, "Frame4" );
   Frame4->setFrameShape( QFrame::Box );
   Frame4->setFrameShadow( QFrame::Sunken );
   Frame4Layout = new QVBoxLayout( Frame4 ); 
   Frame4Layout->setSpacing( 6 );
   Frame4Layout->setMargin( 11 );

//   code_v = new QTextView( Frame4, "code_v" );
   code_v = new QTextBrowser( Frame4, "code_v" );
//   code_v->setEnabled( FALSE );
   code_v->setText( tr( "this is a test for code.  one two three four." ) );
//   code_v->setWordWrap( QMultiLineEdit::WidgetWidth );
//   code_v->setWrapPolicy( QMultiLineEdit::AtWhiteSpace );
   Frame4Layout->addWidget( code_v );

   edit_v = new QMultiLineEdit( Frame4, "edit_v" );
   edit_v->setText( tr( "this is a test for gode. one two three fowl." ) );
   edit_v->setWordWrap( QMultiLineEdit::WidgetWidth );
   edit_v->setWrapPolicy( QMultiLineEdit::AtWhiteSpace );
   Frame4Layout->addWidget( edit_v );
   ProfLayout->addWidget( Frame4 );

   Layout8 = new QHBoxLayout; 
   Layout8->setSpacing( 6 );
   Layout8->setMargin( 0 );

   start_b = new QPushButton( this, "start_b" );
   start_b->setText( tr( "Start" ) );
   Layout8->addWidget( start_b );

   stop_b = new QPushButton( this, "stop_b" );
   stop_b->setText( tr( "Stop" ) );
   Layout8->addWidget( stop_b );

   close_b = new QPushButton( this, "close_b" );
   close_b->setText( tr( "Close" ) );
   Layout8->addWidget( close_b );
   ProfLayout->addLayout( Layout8 );

   elapsed_t = new QTimer(this, "");

   // signals and slots connections
   connect( close_b,   SIGNAL( clicked() ), this, SLOT( close() ) );
   connect( start_b,   SIGNAL( clicked() ), this, SLOT( startPressed() ) );
   connect( stop_b,    SIGNAL( clicked() ), this, SLOT( stopPressed() ) );
   connect( elapsed_t, SIGNAL( timeout() ), this, SLOT( elapsedTimeout() ) );

   m_morse = Morse::getInstance();

   m_mRand     = MRand::getInstance();
   m_accepting = false;
}

/*  
 *  Destroys the object and frees any allocated resources
 */
Prof::~Prof()
{
   // no need to delete child widgets, Qt does it all for us
}

void Prof::closeEvent(QCloseEvent *e)
{
   m_morse->abort();
   elapsed_t->stop();

   emit closed();
   QDialog::closeEvent(e);
}

void Prof::show()
{
   m_morse->abort();
   elapsed_t->stop();

   m_sent      = "";
   m_accepting = false;
   
   time_v->setText("0:00");
   code_v->setText("");
   edit_v->setText("");

   QDialog::show();
}

/*
void Prof::keyPressEvent(QKeyEvent *e)
{
   char ch = e->ascii();
   ch = toupper(ch);

   if ( m_accepting && (ch >= ' ' && ch <='~') )
   {
      QString tempNum;
      if( (char) (QChar) m_sent[m_current] == ch)
      {
         //morse_t->start(5000, TRUE);

         m_correct++;
         tempNum.setNum(m_correct);
         correct_v->setText(tempNum);

         tempNum.sprintf("%.1f%%", m_correct * 100.0 / (m_incorrect + m_correct) );
         percent_v->setText(tempNum);

         m_current++;

         sent_v->setText( tr( sent_v->text().append(QString( tr(" ") + QChar(ch))) ) );

         if(m_sent[m_current] == '\0')
         {
            m_accepting = false;
            m_morse->correct();
            morse_t->stop();
            m_sent = m_mRand->getString();
            resend_t->start(2000, TRUE);
         }
      }
      else
      {
         m_accepting = false;
         morse_t->stop();
         m_morse->incorrect();

         m_incorrect++;
         tempNum.setNum(m_incorrect);
         incorrect_v->setText(tempNum);

         tempNum.sprintf("%.1f%%", m_correct * 100.0 / (m_incorrect + m_correct) );
         percent_v->setText(tempNum);
         
         sent_v->setText( tr( sent_v->text().append(QString( tr(" <font color=\"red\">") + QChar(m_sent[m_current]) + tr("</font>") ) ) ) );
         message_v->setText( tr( "<html><font color=\"red\">") + QChar(ch) + tr("</font>")  );
         m_current = 0;

         resend_t->start(2000, TRUE);
      }
   }
//   else
//   {
      QDialog::keyPressEvent(e);
//   }
}
*/

void Prof::stopPressed(void)
{
   QString show = "<html>";

   const char *code;
   const char *edit;

   int a = 0;
   int b = 0;
   int codelen = 0;
   int editlen = 0;

   int len = 0;
   int longest = 0;

   if ( m_morse->sending() )
   {
      elapsed_t->stop();
      m_morse->abort();
      stop_b->setText("Show answer");
   }
   else
   {
      code = m_sent.latin1();
      codelen = strlen(code);
      QString editstr = edit_v->text();
      edit = editstr.latin1();
      editlen = strlen(edit);

      while ( a < codelen && b < codelen)
      {
         printf("%c\n", code[a]);
         if ( isspace ( code[a] ) )
         {
            show += " ";
            if ( isspace ( edit[b] ) )
            {
               while ( a < codelen && isspace( code[a] ) )
                  a++;
               while ( b < editlen && isspace( edit[b] ) )
                  b++;
            }
            else
            {
               len = 0;
               while ( ! isspace( edit[b] ) )
                  b++;

               while ( a < codelen && isspace( code[a] ) )
                  a++;
               while ( b < editlen && isspace( edit[b] ) )
                  b++;
            }
         }
         else if ( isspace ( edit[b] ) )
         {
            len = 0;
            show += "<font color=\"red\">";
            while ( ! isspace ( code[a] ) )
               show += code[a++];

            show += "</font> ";

            while ( a < codelen && isspace( code[a] ) )
               a++;
            while ( b < editlen && isspace( edit[b] ) )
               b++;
         }
         else
         {
            if ( code[a] == edit[b] )
            {
               show += code[a];

               len++;
               if ( len > longest )
                  longest = len;
               a++;
               b++;
            }
            else
            {
               len = 0;

               int lena = 0;
               int lenb = 0;
               int tempa = a;
               int tempb = b;

               bool corrected = false;

               while ( ! isspace( code[tempa] ) )
               {
                  tempa++;
                  lena++;
               }

               while ( ! isspace( edit[tempb] ) )
               {
                  tempb++;
                  lenb++;
               }

               tempa = a;
               tempb = b;

               while ( lena > lenb )
               {
                  tempa++;
                  lena--;

                  if ( code[tempa] == edit[b] )
                  {
                     show += "<font color=\"red\">";
                     while ( a < tempa )
                        show += code[a++];
                     show += "</font>";

                     show += code[a];
                     corrected = true;
                     len++;
                     if ( len > longest)
                        longest = len;
                     a++;
                     b++;
                  }
               }

               while ( lenb > lena )
               {
                  tempb++;
                  lenb--;

                  if ( code[a] == edit[tempb] )
                  {
                     b = tempb;

                     show += "<font color=\"red\">|</font>";

                     show += code[a];
                     corrected = true;
                     len++;
                     if ( len > longest)
                        longest = len;
                     a++;
                     b++;
                  }
               }

               if ( ! corrected )
               {
                  show += "<font color=\"red\">";
                  show += code[a];
                  show += "</font>";
                  a++;
                  b++;
               }
            }
         }
      }

      if ( a < codelen )
      {
         show += "<font color=\"red\">";
         show += code;
         show += "</font>";
      }

      if ( b < codelen )
      {
         show += "<font color=\"red\">|</font>";
      }

      printf("%s\n", show.latin1());
//      code_v->setText( show );
      code_v->setText( "Doesn't work yet!  =]" );
      m_accepting = false;
   }
}

void Prof::startPressed(void)
{
   m_morse->abort();
   elapsed_t->stop();

   m_sent      = "";
   m_accepting = true;
   
   code_v->setText("");
   edit_v->setText("");

//   char * temp = m_mRand->getString();
   m_sent = "1234. *";
//   free (temp);

   edit_v->setFocus();

   stop_b->setText("Stop");

   m_morse->send( m_sent.latin1() );
   m_startTime = time(NULL);
   elapsed_t->start(1000, FALSE);
}

void Prof::elapsedTimeout(void)
{
   int min, sec, t;
   t = time(NULL) - m_startTime;

   if ( ! m_morse->sending()  ) 
   {
      stop_b->setText("Show answer");
      elapsed_t->stop();
   }

   min = t / 60;
   sec = t % 60;

   QString timestr;
   timestr.sprintf("%d:%02d", min, sec);
   time_v->setText(timestr);
}

void Prof::reject()
{
   emit closed();
   QDialog::reject();
}

