/* recgame.cpp - Morse Code Dojo receiving game
 * 
 *  Morse Code Dojo
 * 
 *  Copyright (c) 2001 Kevin Worcester
 * 
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, 
 *  USA.
 */
#include <qframe.h>
#include <qlabel.h>
#include <qpushbutton.h>
#include <qlayout.h>
#include <qvariant.h>
#include <qtooltip.h>
#include <qwhatsthis.h>
#include <qtimer.h>
#include <ctype.h>

#include "recgame.h"
#include "morse.h"
#include "mrand.h"

/* 
 *  Constructs a RecGame which is a child of 'parent', with the 
 *  name 'name' and widget flags set to 'f' 
 *
 *  The dialog will by default be modeless, unless you set 'modal' to
 *  TRUE to construct a modal dialog.
 */
RecGame::RecGame( QWidget* parent,  const char* name, bool modal, WFlags fl )
   : QDialog( parent, name, modal, fl )
{
   if ( !name )
   setName( "RecGame" );
   resize( 592, 480 ); 
   setCaption( tr( "Receiving Game" ) );
   RecGameLayout = new QVBoxLayout( this ); 
   RecGameLayout->setSpacing( 6 );
   RecGameLayout->setMargin( 11 );

   Frame5 = new QFrame( this, "Frame5" );
   Frame5->setSizePolicy( QSizePolicy( (QSizePolicy::SizeType)5, (QSizePolicy::SizeType)0, Frame5->sizePolicy().hasHeightForWidth() ) );
   Frame5->setFrameShape( QFrame::Box );
   Frame5->setFrameShadow( QFrame::Sunken );
   Frame5Layout = new QVBoxLayout( Frame5 ); 
   Frame5Layout->setSpacing( 6 );
   Frame5Layout->setMargin( 11 );

   Layout5 = new QHBoxLayout; 
   Layout5->setSpacing( 6 );
   Layout5->setMargin( 0 );

   Layout2 = new QHBoxLayout; 
   Layout2->setSpacing( 6 );
   Layout2->setMargin( 0 );

   correct_l = new QLabel( Frame5, "correct_l" );
   correct_l->setText( tr( "Correct:" ) );
   Layout2->addWidget( correct_l );

   correct_v = new QLabel( Frame5, "correct_v" );
   correct_v->setText( tr( "0" ) );
   Layout2->addWidget( correct_v );
   Layout5->addLayout( Layout2 );

   Layout3 = new QHBoxLayout; 
   Layout3->setSpacing( 6 );
   Layout3->setMargin( 0 );

   incorrect_l = new QLabel( Frame5, "incorrect_l" );
   incorrect_l->setText( tr( "Incorrect:" ) );
   Layout3->addWidget( incorrect_l );

   incorrect_v = new QLabel( Frame5, "incorrect_v" );
   incorrect_v->setText( tr( "0" ) );
   Layout3->addWidget( incorrect_v );
   Layout5->addLayout( Layout3 );

   Layout4 = new QHBoxLayout; 
   Layout4->setSpacing( 6 );
   Layout4->setMargin( 0 );

   percent_l = new QLabel( Frame5, "percent_l" );
   percent_l->setText( tr( "Percent:" ) );
   Layout4->addWidget( percent_l );

   percent_v = new QLabel( Frame5, "percent_v" );
   percent_v->setText( tr( "0.0%" ) );
   Layout4->addWidget( percent_v );
   Layout5->addLayout( Layout4 );
   Frame5Layout->addLayout( Layout5 );

   Layout1 = new QHBoxLayout; 
   Layout1->setSpacing( 6 );
   Layout1->setMargin( 0 );

   time_l = new QLabel( Frame5, "time_l" );
   time_l->setText( tr( "Time elapsed:" ) );
   time_l->setAlignment( int( QLabel::AlignVCenter | QLabel::AlignRight ) );
   Layout1->addWidget( time_l );

   time_v = new QLabel( Frame5, "time_v" );
   time_v->setText( tr( "0:00" ) );
   Layout1->addWidget( time_v );
   Frame5Layout->addLayout( Layout1 );
   RecGameLayout->addWidget( Frame5 );

   Frame4 = new QFrame( this, "Frame4" );
   Frame4->setFrameShape( QFrame::Box );
   Frame4->setFrameShadow( QFrame::Sunken );
   Frame4Layout = new QVBoxLayout( Frame4 ); 
   Frame4Layout->setSpacing( 6 );
   Frame4Layout->setMargin( 11 );
   QSpacerItem* spacer = new QSpacerItem( 20, 20, QSizePolicy::Minimum, QSizePolicy::Expanding );
   Frame4Layout->addItem( spacer );

   sent_v = new QLabel( Frame4, "sent_v" );
   sent_v->setSizePolicy( QSizePolicy( (QSizePolicy::SizeType)5, (QSizePolicy::SizeType)5, sent_v->sizePolicy().hasHeightForWidth() ) );
   QFont sent_v_font(  sent_v->font() );
   sent_v_font.setPointSize( 20 );
   sent_v->setFont( sent_v_font ); 
   sent_v->setText( "<html>" );
   sent_v->setAlignment( int( QLabel::AlignCenter ) );
   Frame4Layout->addWidget( sent_v );

   message_v = new QLabel( Frame4, "message_v" );
   QFont message_v_font(  message_v->font() );
   message_v_font.setPointSize( 20 );
   message_v->setFont( message_v_font ); 
   message_v->setText( "<html>" );
   message_v->setAlignment( int( QLabel::AlignCenter ) );
   Frame4Layout->addWidget( message_v );

   QSpacerItem* spacer_2 = new QSpacerItem( 20, 20, QSizePolicy::Minimum, QSizePolicy::Expanding );
   Frame4Layout->addItem( spacer_2 );
   RecGameLayout->addWidget( Frame4 );

   Layout8 = new QHBoxLayout; 
   Layout8->setSpacing( 6 );
   Layout8->setMargin( 0 );

   start_b = new QPushButton( this, "start_b" );
   start_b->setText( tr( "Start" ) );
   Layout8->addWidget( start_b );

   stop_b = new QPushButton( this, "stop_b" );
   stop_b->setText( tr( "Stop" ) );
   Layout8->addWidget( stop_b );

   close_b = new QPushButton( this, "close_b" );
   close_b->setText( tr( "Close" ) );
   Layout8->addWidget( close_b );
   RecGameLayout->addLayout( Layout8 );

   morse_t   = new QTimer(this, "");
   resend_t  = new QTimer(this, "");
   elapsed_t = new QTimer(this, "");

   // signals and slots connections
   connect( close_b,   SIGNAL( clicked() ), this, SLOT( close() ) );
   connect( start_b,   SIGNAL( clicked() ), this, SLOT( startPressed() ) );
   connect( stop_b,    SIGNAL( clicked() ), this, SLOT( stopPressed() ) );
   connect( morse_t,   SIGNAL( timeout() ), this, SLOT( morseTimeout() ) );
   connect( resend_t,  SIGNAL( timeout() ), this, SLOT( resendTimeout() ) );
   connect( elapsed_t, SIGNAL( timeout() ), this, SLOT( elapsedTimeout() ) );

   m_morse = Morse::getInstance();

   m_mRand     = MRand::getInstance();
   m_accepting = false;
   m_maxTime   = 60;
}

RecGame::~RecGame()
{
}

void RecGame::closeEvent(QCloseEvent *e)
{
   m_morse->abort();
   resend_t->stop();
   elapsed_t->stop();
   morse_t->stop();

   emit closed();
   QDialog::closeEvent(e);
}

void RecGame::show()
{
   m_morse->abort();
   morse_t->stop();
   elapsed_t->stop();
   resend_t->stop();

   m_correct   = 0;
   m_incorrect = 0;
   m_current   = 0;
   m_lastWrong = 0;
   m_sent      = "";
   m_max       = -1;
   m_accepting = false;
   
   correct_v->setText("0");
   incorrect_v->setText("0");
   percent_v->setText("0.0%");
   time_v->setText("0:00");
   sent_v->setText("<html>");
   message_v->setText("<html>");

   QDialog::show();
}

/*  
 *  Main event handler. Reimplemented to handle application
 *  font changes
 */
bool RecGame::event( QEvent* ev )
{
   bool ret = QDialog::event( ev ); 
   if ( ev->type() == QEvent::ApplicationFontChange ) {
      QFont sent_v_font(  sent_v->font() );
      sent_v_font.setPointSize( 20 );
      sent_v->setFont( sent_v_font ); 
      QFont message_v_font(  message_v->font() );
      message_v_font.setPointSize( 20 );
      message_v->setFont( message_v_font ); 
   }
   return ret;
}

void RecGame::keyPressEvent(QKeyEvent *e)
{
   char ch = e->ascii();
   ch = toupper(ch);

   if ( m_accepting && (ch >= ' ' && ch <='~') )
   {
      QString tempNum;
      if( (char) (QChar) m_sent[m_current] == ch)
      {
         //morse_t->start(5000, TRUE);

         m_correct++;
         tempNum.setNum(m_correct);
         correct_v->setText(tempNum);

         tempNum.sprintf("%.1f%%", m_correct * 100.0 / (m_incorrect + m_correct) );
         percent_v->setText(tempNum);

         m_current++;

         sent_v->setText( tr( sent_v->text().append(QString( tr(" ") + QChar(ch))) ) );

         if(m_sent[m_current] == '\0')
         {
            m_accepting = false;
            m_morse->correct();
            morse_t->stop();
            m_sent = m_mRand->getString();
            resend_t->start(2000, TRUE);
         }
      }
      else
      {
         m_accepting = false;
         morse_t->stop();
         m_morse->incorrect();

         m_incorrect++;
         tempNum.setNum(m_incorrect);
         incorrect_v->setText(tempNum);

         tempNum.sprintf("%.1f%%", m_correct * 100.0 / (m_incorrect + m_correct) );
         percent_v->setText(tempNum);
         
         sent_v->setText( tr( sent_v->text().append(QString( tr(" <font color=\"red\">") + QChar(m_sent[m_current]) + tr("</font>") ) ) ) );
         message_v->setText( tr( "<html><font color=\"red\">") + QChar(ch) + tr("</font>")  );
         m_current = 0;

         resend_t->start(2000, TRUE);
      }
   }
   else
   {
      QDialog::keyPressEvent(e);
   }
}

void RecGame::stopPressed(void)
{
   m_morse->abort();
   morse_t->stop();
   elapsed_t->stop();
   resend_t->stop();
   m_accepting = false;
}

void RecGame::startPressed(void)
{
   m_morse->abort();
   morse_t->stop();
   elapsed_t->stop();
   resend_t->stop();

   m_correct   = 0;
   m_incorrect = 0;
   m_current   = 0;
   m_lastWrong = 0;
   m_sent      = "";
   m_max       = -1;
   m_accepting = true;
   
   correct_v->setText("0");
   incorrect_v->setText("0");
   sent_v->setText("<html>");
   message_v->setText("<html>");

   char * temp = m_mRand->getString();
   m_sent = temp;
   free (temp);

   m_morse->send( m_sent.latin1() );
//   morse_t->start(5000, TRUE);
   m_startTime = time(NULL);
   elapsed_t->start(1000, FALSE);
}

void RecGame::morseTimeout(void)
{
   m_morse->incorrect();
   m_accepting = false;

   QString tempNum;

   m_incorrect++;
   tempNum.setNum(m_incorrect);
   incorrect_v->setText(tempNum);

   tempNum.sprintf("%.1f%%", m_correct * 100.0 / (m_incorrect + m_correct) );
   percent_v->setText(tempNum);
   
   sent_v->setText( tr( sent_v->text().append(QString( tr(" <font color=\"red\">") + QChar(m_sent[m_current]) + tr("</font>") ) ) ) );
   message_v->setText( tr( "<html><font color=\"red\">Timeout!</font>" ) );

   m_current = 0;

   resend_t->start(3000, TRUE);
}

void RecGame::resendTimeout(void)
{
   m_current   = 0;
   m_lastWrong = 0;
   m_max       = -1;
   m_accepting = true;
   
   sent_v->setText("<html>");
   message_v->setText("<html>");

   m_morse->send( m_sent.latin1() );
   morse_t->stop();
//   morse_t->start(5000, TRUE);
}

void RecGame::elapsedTimeout(void)
{
   int min, sec, t;
   t = time(NULL) - m_startTime;

   if ( ! m_morse->sending() && ! morse_t->isActive() && m_accepting == true ) 
   {
      morse_t->start(3000, TRUE);
   }

   if ( t >= m_maxTime )
   {
      elapsed_t->stop();
      resend_t->stop();
      morse_t->stop();

      m_morse->abort();
      m_accepting = false;

      message_v->setText("Time up");
   }
   min = t / 60;
   sec = t % 60;

   QString timestr;
   timestr.sprintf("%d:%02d", min, sec);
   time_v->setText(timestr);
}

void RecGame::setTime(time_t t)
{
   m_maxTime = t;
}

time_t RecGame::getTime(void)
{
   return m_maxTime;
}

