/* recgame.h - Morse Code Dojo receiving game
 * 
 *  Morse Code Dojo
 * 
 *  Copyright (c) 2001 Kevin Worcester
 * 
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, 
 *  USA.
 */
#ifndef RECGAME_H
#define RECGAME_H

#include <qvariant.h>
#include <qdialog.h>
#include <time.h>
#include <sys/types.h>

class QVBoxLayout; 
class QHBoxLayout; 
class QGridLayout; 
class QFrame;
class QLabel;
class QPushButton;
class QTimer;
class Morse;
class MRand;

class RecGame : public QDialog
{ 
   Q_OBJECT

public:
   RecGame( QWidget* parent = 0, const char* name = 0, bool modal = FALSE, WFlags fl = 0 );
   virtual ~RecGame();
   void show();

   void   setTime(time_t t);
   time_t getTime();

protected slots:
   void startPressed();
   void stopPressed();
   void morseTimeout();
   void resendTimeout();
   void elapsedTimeout();
   void closeEvent(QCloseEvent *);

signals:
   void closed();

protected:
   void keyPressEvent(QKeyEvent *e);
   int m_correct;
   int m_incorrect;
   int m_current;
   int m_max;
   int m_lastWrong;
   QString m_sent;
   Morse *m_morse;
   MRand *m_mRand;
   time_t m_startTime;
   time_t m_maxTime;
   bool   m_accepting; /* Accepting keystrokes? */

   QFrame * Frame5;
   QLabel * correct_l;
   QLabel * correct_v;
   QLabel * incorrect_l;
   QLabel * incorrect_v;
   QLabel * percent_l;
   QLabel * percent_v;
   QLabel * time_l;
   QLabel * time_v;
   QFrame * Frame4;
   QLabel * sent_v;
   QLabel * message_v;

   QTimer      * morse_t;
   QTimer      * resend_t;
   QTimer      * elapsed_t;
   QPushButton * start_b;
   QPushButton * stop_b;
   QPushButton * close_b;
   QVBoxLayout * RecGameLayout;
   QVBoxLayout * Frame5Layout;
   QHBoxLayout * Layout5;
   QHBoxLayout * Layout2;
   QHBoxLayout * Layout3;
   QHBoxLayout * Layout4;
   QHBoxLayout * Layout1;
   QVBoxLayout * Frame4Layout;
   QHBoxLayout * Layout8;
   bool event( QEvent* );
};

#endif // RECGAME_H
