/* rprop.cpp - Morse Code Dojo Receving Game properties
 * 
 *  Morse Code Dojo
 * 
 *  Copyright (c) 2001 Kevin Worcester
 * 
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, 
 *  USA.
 */
#include "rprop.h"

#include <qcombobox.h>
#include <qlabel.h>
#include <qlineedit.h>
#include <qpushbutton.h>
#include <qspinbox.h>
#include <qlayout.h>
#include <qvariant.h>
#include <qtooltip.h>
#include <qwhatsthis.h>
#include <qregexp.h>
#include <ctype.h>
#include <stdio.h>
#include <stdlib.h>

/* 
 *  Constructs a RecProperties which is a child of 'parent', with the 
 *  name 'name' and widget flags set to 'f' 
 *
 *  The dialog will by default be modeless, unless you set 'modal' to
 *  TRUE to construct a modal dialog.
 */
RecProperties::RecProperties( QWidget* parent,  const char* name, bool modal, WFlags fl )
   : QDialog( parent, name, modal, fl )
{
   if ( !name )
   setName( "RecProperties" );
   resize( 266, 224 ); 
   setCaption( tr( "Morse Code Dojo" ) );
   RecPropertiesLayout = new QGridLayout( this ); 
   RecPropertiesLayout->setSpacing( 6 );
   RecPropertiesLayout->setMargin( 11 );

   char_l = new QLabel( this, "char_l" );
   char_l->setText( tr( "Character Speed:" ) );

   RecPropertiesLayout->addWidget( char_l, 0, 0 );

   char_v = new QSpinBox( this, "char_v" );
   char_v->setMaxValue( 30 );
   char_v->setMinValue( 5 );
   char_v->setValue( 17 );

   RecPropertiesLayout->addWidget( char_v, 0, 1 );

   word_l = new QLabel( this, "word_l" );
   word_l->setText( tr( "Word Speed:" ) );

   RecPropertiesLayout->addWidget( word_l, 1, 0 );

   word_v = new QSpinBox( this, "word_v" );
   word_v->setMaxValue( 30 );
   word_v->setMinValue( 5 );
   word_v->setValue( 8 );

   RecPropertiesLayout->addWidget( word_v, 1, 1 );

   min_l = new QLabel( this, "min_l" );
   min_l->setText( tr( "Min Word Length:" ) );

   RecPropertiesLayout->addWidget( min_l, 2, 0 );

   min_v = new QSpinBox( this, "min_v" );
   min_v->setMaxValue( 10 );
   min_v->setMinValue( 3 );
   min_v->setValue( 5 );

   RecPropertiesLayout->addWidget( min_v, 2, 1 );

   max_l = new QLabel( this, "max_l" );
   max_l->setText( tr( "Max Word Length:" ) );

   RecPropertiesLayout->addWidget( max_l, 3, 0 );

   max_v = new QSpinBox( this, "max_v" );
   max_v->setMaxValue( 10 );
   max_v->setMinValue( 3 );
   max_v->setValue( 5 );

   RecPropertiesLayout->addWidget( max_v, 3, 1 );

   time_l = new QLabel( this, "time_l" );
   time_l->setText( tr( "Time:" ) );

   RecPropertiesLayout->addWidget( time_l, 4, 0 );

   time_v = new QLineEdit( this, "time_v" );
   time_v->setText( tr( "3:00" ) );
   time_v->setMaxLength( 6 );

   RecPropertiesLayout->addWidget( time_v, 4, 1 );

   weight_l = new QLabel( this, "weight_l" );
   weight_l->setText( tr( "Weighting" ) );

   RecPropertiesLayout->addWidget( weight_l, 5, 0 );

   weight_v = new QComboBox( FALSE, this, "weight_v" );
   weight_v->insertItem( tr( "Random" ) );
   weight_v->insertItem( tr( "User" ) );
   weight_v->insertItem( tr( "History" ) );

   RecPropertiesLayout->addWidget( weight_v, 5, 1 );

   ok_b = new QPushButton( this, "ok_b" );
   ok_b->setText( tr( "Ok" ) );
   ok_b->setDefault( TRUE );

   RecPropertiesLayout->addWidget( ok_b, 6, 0 );

   cancel_b = new QPushButton( this, "cancel_b" );
   cancel_b->setText( tr( "Cancel" ) );

   RecPropertiesLayout->addWidget( cancel_b, 6, 1 );

   // signals and slots connections
   connect( cancel_b, SIGNAL( clicked() ), this, SLOT( reject() ) );
   connect( ok_b, SIGNAL( clicked() ), this, SLOT( okClicked() ) );
   connect( word_v, SIGNAL( valueChanged(int) ), this, SLOT( wordSpeedChanged(int) ) );
   connect( char_v, SIGNAL( valueChanged(int) ), this, SLOT( charSpeedChanged(int) ) );
   connect( min_v, SIGNAL( valueChanged(int) ), this, SLOT( minLengthChanged(int) ) );
   connect( max_v, SIGNAL( valueChanged(int) ), this, SLOT( maxLengthChanged(int) ) );
//   connect( ok_b, SIGNAL( clicked() ), this, SLOT( accept() ) );
}

/*  
 *  Destroys the object and frees any allocated resources
 */
RecProperties::~RecProperties()
{
   // no need to delete child widgets, Qt does it all for us
}

void RecProperties::setCharSpeed(int s)
{
   char_v->setValue(s);
}

void RecProperties::setWordSpeed(int s)
{
   word_v->setValue(s);
}

int RecProperties::getCharSpeed(void)
{
   return char_v->value();
}

int RecProperties::getWordSpeed(void)
{
   return word_v->value();
}

void RecProperties::setMinLength(int l)
{
   min_v->setValue(l);
}

void RecProperties::setMaxLength(int l)
{
   max_v->setValue(l);
}

int RecProperties::getMinLength(void)
{
   return min_v->value();
}

int RecProperties::getMaxLength(void)
{
   return max_v->value();
}

void RecProperties::setTime(time_t t)
{
   QString s;
   s.sprintf( "%d:%02d", (int) t / 60, (int) t % 60 );

   time_v->setText(s);
}

time_t RecProperties::getTime(void)
{
   QString s;

   s = time_v->text();
   QRegExp r("[0-9]:[0-5][0-9]");
   
   if( r.find(s, 0) < 0 )
   {
      printf("Time must be of the format 'm:ss'\n");
      return 0;
   }

   return stringToTime(time_v->text());
}

void RecProperties::okClicked(void)
{
   if ( char_v->value() < word_v->value() )
   {
      printf("Character speed must be equal to or greater than word speed\n");
      return;
   }

   QString s = time_v->text();
   QRegExp r("[0-9]:[0-5][0-9]");
   
   if( r.find(s, 0) < 0 )
   {
      printf("Time must be of the format 'm:ss'\n");
      return;
   }

   time_t t = stringToTime(time_v->text());
   if ( t <= 60 )
   {
      printf("Must be at least a minute");
   }

   emit accept();
}

time_t RecProperties::stringToTime(QString s)
{
   time_t t;

   QRegExp r("[0-9]:[0-5][0-9]");
   if( r.find(s, 0) < 0 )
   {
      return 0;
   }

   t = (atoi(s.latin1()) * 60) + (atoi( (s.right(2)).latin1() ));

   return t;
}

void RecProperties::wordSpeedChanged(int s)
{
   if ( word_v->value() > char_v->value() )
   {
      char_v->setValue( word_v->value() );
   }
}

void RecProperties::charSpeedChanged(int s)
{
   if ( word_v->value() > char_v->value() )
   {
      word_v->setValue( char_v->value() );
   }
}

void RecProperties::maxLengthChanged(int s)
{
   if ( min_v->value() > max_v->value() )
   {
      min_v->setValue( max_v->value() );
   }
}

void RecProperties::minLengthChanged(int s)
{
   if ( min_v->value() > max_v->value() )
   {
      max_v->setValue( min_v->value() );
   }
}

